/*
* Copyright (c) 2014 Christian Surlykke
*
* This file is part of the Nde project. <https://lxqt.org>
* It is distributed under the LGPL 2.1 or later license.
* Please refer to the LICENSE file for a copy of the license.
*/

#include "batteryinfodialog.h"
#include "ui_batteryinfodialog.h"
#include "batteryhelper.h"
#include <Nde/Globals>
#include <kwindoweffects.h>
#include <QFormLayout>
#include <QTabWidget>
#include <QApplication>
#include <QDesktopWidget>
#include <QDebug>
#include <QCursor>
#include <QBitmap>
#include <QString>
#include <QPainter>
#include <QDBusConnection>
#include <QDBusInterface>

BatteryInfoDialog::BatteryInfoDialog(QList<Solid::Battery*> batteries, QWidget *parent) :
    QDialog(parent),
    ui(new Ui::BatteryInfoDialog)
{
    ui->setupUi(this);

    setWindowTitle(tr("Battery Info"));

    if (batteries.size() == 1)
    {
        BatteryInfoFrame *batteryInfoFrame = new BatteryInfoFrame(batteries[0]);
        ui->verticalLayout->insertWidget(0, batteryInfoFrame);
    }
    else
    {
        QTabWidget *tabWidget = new QTabWidget(this);
        ui->verticalLayout->insertWidget(0, tabWidget);
        for (Solid::Battery *const battery : qAsConst(batteries))
        {
            BatteryInfoFrame *batteryInfoFrame = new BatteryInfoFrame(battery);
            tabWidget->addTab(batteryInfoFrame, QSL("BAT"));
        }
    }
}

BatteryInfoDialog::~BatteryInfoDialog()
{
    delete ui;
}

void BatteryInfoDialog::toggleShow()
{
    qDebug() << "toggleShow";
    isVisible() ? hide() : show();
}

BatteryWidget::BatteryWidget(QList<Solid::Battery*> batteries,QWidget* parent)
    : QWidget(parent)
{
    battery = batteries;
    this->setObjectName("BatteryWidget");
    this->setWindowFlags(Qt::FramelessWindowHint |
                         Qt::Tool |
                         Qt::X11BypassWindowManagerHint);
    this->setAttribute(Qt::WA_TranslucentBackground);
    this->setAttribute(Qt::WA_NoSystemBackground, false);

    KWindowEffects::enableBlurBehind(this->winId(), true);

    QDBusConnection::sessionBus().connect(QString::fromLatin1("org.nde.session"),
                                          QString::fromLatin1("/NdeSession"),
                                          QString::fromLatin1("org.nde.session"),
                                          QString::fromLatin1("compositorOpenStateChanged"),
                                          this, SLOT(onCompositorOpenChanged(bool)));

    QDBusInterface * compositer = new QDBusInterface(QString::fromLatin1("org.nde.session"),
                                                     QString::fromLatin1("/NdeSession"),
                                                     QString::fromLatin1("org.nde.session"),
                                                     QDBusConnection::sessionBus());
    if(compositer->isValid())
    {
        QDBusMessage reply = compositer->call("getCompositorOpen");
        if(!reply.arguments().isEmpty())
        {
            mCompositorOpen = reply.arguments().first().toBool();
            this->setProperty("compositor", mCompositorOpen);
        }
    }

    setFixedSize(180,50);
    Init();
}
void BatteryWidget::paintEvent(QPaintEvent * event)
{
    QRect rect = this->rect().adjusted(0,0,-1,-1);
    QBitmap bmp(rect.size());
    bmp.fill();
    QPainter painter(&bmp);
    painter.setBrush(Qt::black);
    painter.setRenderHints(QPainter::HighQualityAntialiasing );
    painter.drawRoundedRect(bmp.rect(),3,3);
    setMask(bmp);
}

bool BatteryWidget::event(QEvent *e)
{
    if(e->type() == QEvent::ActivationChange)
    {
        if(!this->isActiveWindow())
        {
            this->hide();
        }
    }
    return QWidget::event(e);
}

void BatteryWidget::Init()
{
    icon =new QLabel();
    icon->setFixedSize(40,20);

    cout = new QLabel();
    cout->setObjectName("label_cout");
    //cout->setFixedWidth(50);;
    cout->setAlignment(Qt::AlignLeft);
    msg = new QLabel();
    msg->setObjectName("label_msg");
    msg->setAlignment(Qt::AlignLeft);
    lay = new QHBoxLayout();

    lay->addSpacing(10);
    lay->addWidget(icon);
    lay->addSpacing(5);
    lay->addWidget(cout);
    lay->addSpacing(5);
    lay->addWidget(msg);
    lay->addStretch();
    setLayout(lay);
}

void BatteryWidget::update()
{
    int ChargePercent = BatteryHelper::getBatteryChargecent();
    Solid::Battery::ChargeState state = BatteryHelper::getBatteryState();
    QPixmap pixmap = icontool.genIcon(ChargePercent,state);

    icon->setPixmap(pixmap.scaled(40,20, Qt::IgnoreAspectRatio));
    cout->setText(QString::number(ChargePercent) + "%");
    if(state == Solid::Battery::Charging){
        msg->setText(tr("Charging"));
    } else if( state ==  Solid::Battery::Discharging){
        if (ChargePercent <= 20) {
            msg->setText(tr("Empty"));
        } else {
            msg->setText(tr("Battery"));
        }
    } else {
        msg->setText(tr("Battery"));
    }
}

void BatteryWidget::toggleShow()
{
    activateWindow();
    update();
    QCursor cur;
    QRect rect = QApplication::desktop()->screenGeometry();

    Nde::Settings setting(QLatin1String("panel"),this);
    setting.beginGroup("panel1");
    QString position = setting.value("position").toString();
    int panelwidth =  setting.value("panelSize").toInt();
    setting.endGroup();

    if(position.compare("Bottom") == 0) {
        move(cur.pos().x()-width()/2,rect.height()-panelwidth - height()-8);
    } else if (position.compare("Left") == 0) {
        move(panelwidth + 8,cur.pos().y()-height()/2);
    } else if (position.compare("Right") == 0) {
        move(rect.width()-panelwidth-width()-8,cur.pos().y()-height()/2);
    } else {
        move(cur.pos().x()-width()/2,panelwidth + 8);
    }

    this->setProperty("compositor", mCompositorOpen);
    this->style()->unpolish(this);
    this->style()->polish(this);

    isVisible() ? hide() : show();
}

void BatteryWidget::onCompositorOpenChanged(bool open)
{
    if(open == mCompositorOpen) return;

    mCompositorOpen = open;
    this->setProperty("compositor", mCompositorOpen);
    this->style()->unpolish(this);
    this->style()->polish(this);
}

