/* BEGIN_COMMON_COPYRIGHT_HEADER
 * (c)LGPL2+
 *
 * Nde - a lightweight, Qt based, desktop toolset
 * https://lxqt.org
 *
 * Copyright: 2012 Razor team
 * Authors:
 *   Christian Surlykke <christian@surlykke.dk>
 *
 * This program or library is free software; you can redistribute it
 * and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA
 *
 * END_COMMON_COPYRIGHT_HEADER */

#include <QDebug>
#include <QTime>
#include <QTimer>

#include <Nde/Notification>
#include <Nde/AutostartEntry>
#include <Nde/Globals>
#include <Nde/Power>
#include <Solid/Device>
#include <Solid/Battery>
#include <QApplication>
#include <QDesktopWidget>
#include "batterywatcher.h"
#include "batteryhelper.h"
#include "../config/powermanagementsettings.h"

BatteryWatcher::BatteryWatcher(QObject *parent) : Watcher(parent),
    mRelayoutTimer(nullptr),
    isCancelAction(false)
{
    const QList<Solid::Device> devices = Solid::Device::listFromType(Solid::DeviceInterface::Battery, QString());
    // if (devices.isEmpty())
    // {
    //     Nde::Notification::notify(tr("No battery!"),
    //             tr("Nde could not find data about any battery - monitoring disabled"),
    //             QSL("preferences-system-power-management"));
    // }

    for (Solid::Device device : devices)
    {
        Solid::Battery *battery = device.as<Solid::Battery>();
        if (battery->type() != Solid::Battery::PrimaryBattery)
        {
            continue;
        }
        mBatteries << battery;
        connect(battery, &Solid::Battery::energyChanged, this, &BatteryWatcher::batteryChanged);
        connect(battery, &Solid::Battery::chargeStateChanged, this, &BatteryWatcher::batteryChanged);
    }

    mBatteryInfoDialog = new BatteryInfoDialog(mBatteries);
    BatteryHelper::batterys = mBatteries;
    mbatterywidget = new BatteryWidget(mBatteries);

    connect(&mSettings, SIGNAL(settingsChanged()), this, SLOT(settingsChanged()));
    connect(Nde::Settings::globalSettings(), SIGNAL(iconThemeChanged()), this, SLOT(settingsChanged()));

    settingsChanged();
    batteryChanged();

    mNotifyWidget = new NotifyWidget();
    mNotifyWidget->hide();
    connect(mNotifyWidget, &NotifyWidget::cancelAction, this, &BatteryWatcher::onCancelAction);

    mOldPowerLowLevel = mSettings.getPowerLowLevel();
    mOldPowerLowAction = mSettings.getPowerLowAction();
    mOldPowerLowWarningTime = mSettings.getPowerLowWarningTime();
}

BatteryWatcher::~BatteryWatcher()
{
    if(mRelayoutTimer)
    {
        mRelayoutTimer->stop();
        delete mRelayoutTimer;
    }
    delete mNotifyWidget;
    mNotifyWidget = nullptr;
}

void BatteryWatcher::batteryChanged()
{
    mRelayoutTimer = new QTimer();
    mRelayoutTimer->setSingleShot(true);
    connect(mRelayoutTimer, &QTimer::timeout, this, &BatteryWatcher::relayoutWaring);
    mRelayoutTimer->start(1000);
}

void BatteryWatcher::settingsChanged()
{
    if (!mSettings.isShowIcon())
    {
        while (!mTrayIcons.isEmpty())
        {
            mTrayIcons.first()->hide();
            mTrayIcons.takeFirst()->deleteLater();
        }
    }
    else if (mTrayIcons.isEmpty())
    {

        if (!mBatteries.empty()){
            mTrayIcons.append(new TrayIcon(mBatteries[0], this));
            connect(mTrayIcons.last(), SIGNAL(toggleShowInfo()), mbatterywidget, SLOT(toggleShow()));
            mTrayIcons.last()->show();
        }
    }
}

void BatteryWatcher::relayoutWaring()
{
    static QTime actionTime;
//    static Nde::Notification *notification = nullptr;

    double totalEnergyFull = 0;
    double totalEnergyNow = 0;
    bool discharging = true;
    double chargeLevel;

    for (const Solid::Battery *battery : qAsConst(mBatteries))
    {
        totalEnergyFull += battery->energyFull();
        totalEnergyNow += battery->energy();
        discharging &= (battery->chargeState() == Solid::Battery::Discharging);
    }
    if(!discharging || mOldPowerLowLevel != mSettings.getPowerLowLevel() || 
        mOldPowerLowAction != mSettings.getPowerLowAction() || mOldPowerLowWarningTime != mSettings.getPowerLowWarningTime())
    {
        mNotifyWidget->hide();
        isCancelAction = false;
        mOldPowerLowLevel = mSettings.getPowerLowLevel();
        mOldPowerLowAction = mSettings.getPowerLowAction();
        mOldPowerLowWarningTime = mSettings.getPowerLowWarningTime();
    }

    chargeLevel = 100 * totalEnergyNow / totalEnergyFull;

    qDebug() <<  "BatteryChanged"
             <<  "discharging:" << discharging
             <<  "chargeLevel:" << chargeLevel
             <<  "actionTime:"  << actionTime;

    bool powerLowActionRequired =
            discharging &&
            chargeLevel < mSettings.getPowerLowLevel();

    if (powerLowActionRequired && !isCancelAction)
    {
        if (actionTime.isNull())
            actionTime = QTime::currentTime().addMSecs(mSettings.getPowerLowWarningTime() * 1000);

//        if (!notification)
//        {
//            notification = new Nde::Notification(tr("Power low!"), this);
//            notification->setTimeout(2000);

//        }

        int milliSecondsToAction = QTime::currentTime().msecsTo(actionTime);

        if (milliSecondsToAction > 0)
        {
            int secondsToAction = milliSecondsToAction / 1000;
            switch (mSettings.getPowerLowAction())
            {
            case -2:
                mNotifyWidget->updateShow(tr("Lock screen"), tr("Lock screen in %n second(s)", nullptr, secondsToAction).arg(secondsToAction));
                break;
            case Nde::Power::PowerSuspend:
                mNotifyWidget->updateShow(tr("Suspending"), tr("Suspending in %n second(s)", nullptr, secondsToAction).arg(secondsToAction));
                //notification->setBody(tr("Suspending in %n second(s)", nullptr, secondsToAction).arg(secondsToAction));
                break;
            case Nde::Power::PowerHibernate:
                mNotifyWidget->updateShow(tr("Hibernating"), tr("Hibernating in %n second(s)", nullptr, secondsToAction).arg(secondsToAction));
                //notification->setBody(tr("Hibernating in %n second(s)", nullptr, secondsToAction).arg(secondsToAction));
                break;
            case Nde::Power::PowerShutdown:
                mNotifyWidget->updateShow(tr("Shutting down"), tr("Shutting down in %n second(s)", nullptr, secondsToAction).arg(secondsToAction));
                //notification->setBody(tr("Shutting down in %n second(s)", nullptr, secondsToAction).arg(secondsToAction));
                break;
            case Nde::Power::PowerMonitorOff:
                mNotifyWidget->updateShow(tr("Power monitor off"), tr("Power monitor off in %n second(s)", nullptr, secondsToAction).arg(secondsToAction));
                break;
            default:
                mNotifyWidget->updateShow();
//                notification->setBody(tr(""));
//                notification->update();
                return;
            }

//            notification->update();

            QTimer::singleShot(200, this, SLOT(batteryChanged()));
        }
        else
        {
            doAction(mSettings.getPowerLowAction());
            mNotifyWidget->hide();

            if (!actionTime.isNull())
                actionTime = QTime();
        }
    }
    else
    {
        if (!actionTime.isNull())
            actionTime = QTime();

//        if (notification)
//        {
//            delete notification;
//            notification = nullptr;
//        }
    }

    if(mbatterywidget)
    {
        mbatterywidget->hide();
//        mbatterywidget->deleteLater();
//        mbatterywidget = nullptr;
    }
}

void BatteryWatcher::onCancelAction()
{
    isCancelAction = true;
}
